{ boardName
}:

{ stdenv
, lib
, libftdi1
, hostname
, git
, pkg-config
, arm-none-eabi-gcc
, ncurses
, fetchgit
, buildPackages
}:

let makeTarget = "build/${boardName}/ec.bin";

in stdenv.mkDerivation (finalAttrs: {
  pname = "chromebook-ec-firmware";
  version = "${boardName}-${finalAttrs.src.rev}";

  src = fetchgit {
    url = "https://chromium.googlesource.com/chromiumos/platform/ec";
    rev = "90210ee80a286c4878652cce69c205af44cd84ed";
    hash = "sha256-IcS94qDCBLOtKeEQixGcOBqxzCH5oebFCFCJoTZEoAA=";
  };

  # ecfw has the same obnoxious run-git-on-myself habit that coreboot does
  getVersion = ''
    #!${buildPackages.runtimeShell}
    cat <<\EOF
    #define CROS_EC_VERSION "${finalAttrs.version} ${placeholder "out"}"
    /* limited to 32 characters */
    #define CROS_EC_VERSION32 "${finalAttrs.version}"
    #define VERSION "${finalAttrs.version}"
    #define BUILDER "nix"
    #define DATE "1970-01-01 00:00:00"
    EOF
  '';

  postPatch = ''
    patchShebangs .
    mv $getVersionPath util/getversion.sh
    chmod +x util/getversion.sh
  '';

  nativeBuildInputs = [
    arm-none-eabi-gcc

    # ecfw uses this to search for libftdi1, which is used in a tool
    # (`ecst`) which runs at build-time (?!?!)
    buildPackages.pkg-config
  ];
  buildInputs = [
    ncurses libftdi1  # part of `ecst`, which runs at build-time
  ];
  enableParallelBuilding = true;

  makeFlags = [
    "BOARD=${boardName}"

    # crusty old codebase, modern compiler
    "EXTRA_CFLAGS=-w"

    # Makefile gets confused when nixpkgs sets $out
    "out=build/${boardName}"

    "CPP=arm-none-eabi-cpp"
    "BUILDCC=${buildPackages.stdenv.cc}/bin/cc"

    #"V=1"

    makeTarget
  ];

  passAsFile = [ "getVersion" ];

  installPhase = ''
    mkdir -p $out/bin
    mv ${makeTarget} $out/ecfw.bin
  '';

  meta = with lib; {
    description = "Firmware for Chromebooks' Embedded Controller (EC)";
  };
})
